@@ -10,11 +10,12 @@ from pygments import highlight
 from .meta import safe_yaml, extract_metadata
 from .md_h import text_to_table, text_to_flow
 from .lexers import JadeLexer
-from .util import string_types, smart_tag_style_for_block_html
+from .util import string_types, smart_tag_style_for_block_html, NAMED_COLORS_AND_BACKGROUND
 from .fix import fix_images_in_markdown, fix_relative_image_path
 
 from .more.auto_fix import fix_content_before_compile
 from .more.md_block import get_md_blocks_info, collect_sub_html_blocks, get_wrap_html_for_markdown_render
+from .wiki_link_syntax import compile_markdown_wiki_link_syntax
 
 import shortuuid
 import hashlib
@@ -36,7 +37,13 @@ HOEDOWN_BLOCK_TYPES = {
     10: 'paragraph',
 }
 
-
+def pick_int(value, default_if_fail=0):
+    # 从字符串中, 获得一个整数值, 如果其中有包含的还
+    if isinstance(value, string_types):
+        s = re.search(r'-?\d+', value)
+        if s:
+            return int(s.group())
+    return default_if_fail
 
 # for both 2.x & 3.x
 try: # python2.x
@@ -128,7 +135,7 @@ PhpLexer.__init__ = patched_php_lexer_init
 
 
 # misaka utils
-BASIC_MISAKA_EXTENSIONS = m.EXT_TABLES | m.EXT_FENCED_CODE | m.EXT_STRIKETHROUGH
+BASIC_MISAKA_EXTENSIONS = m.EXT_TABLES | m.EXT_FENCED_CODE | m.EXT_STRIKETHROUGH | m.EXT_MATH
 
 # 禁用缩进产生的代码块 &  __xx__ 这个语法
 BETTER_MISAKA_EXTENSIONS = BASIC_MISAKA_EXTENSIONS | m.EXT_NO_INTRA_EMPHASIS | m.EXT_DISABLE_INDENTED_CODE
@@ -178,10 +185,12 @@ def make_line_chars_as_position_points(html_content, force=False):
 
 class FarBoxRenderer(m.HtmlRenderer):
     def __init__(self, flags=0, nesting_level=0, is_code_file=False,
-                 raw_content='', compute_md_blocks=False, root_path='', filepath='', for_local=False, fix_image_paths=True):
+                 raw_content='', compute_md_blocks=False, root_path='', filepath='',
+                 for_local=False, fix_image_paths=True, metadata=None):
         m.HtmlRenderer.__init__(self, flags, nesting_level)
         self.is_code_file = is_code_file
 
+        self.metadata = metadata or {} # 头部 metadata 的声明
         self.root_path = root_path
         self.filepath = filepath
         self.for_local = for_local
@@ -196,7 +205,9 @@ class FarBoxRenderer(m.HtmlRenderer):
         self.used_block_length = 0 # for bytes
         self.block_length = 0 # for unicode
         self.html_length = 0 # 已经获得的  html length
+
         self.toc_i = 0
+
         self.toc_html_parts = []
 
         # every block is dict type Data, dict(text, html, text_end_position, html_end_position)
@@ -208,6 +219,11 @@ class FarBoxRenderer(m.HtmlRenderer):
         self.is_first_header = True
 
     def header(self, content, level):
+        hash_name = ""
+        hash_name_c = re.search("^#.*?[^#](#[0-9\w]+$)", content, flags=re.I)
+        if hash_name_c:
+            hash_name = hash_name_c.group(1).lstrip("#")
+            content = re.sub("(#[0-9\w]+$)", "", content, flags=re.I)
         h_class_name = 'h16'
         if self.is_first_header:
             h_class_name += ' md_first_h'
@@ -216,10 +232,13 @@ class FarBoxRenderer(m.HtmlRenderer):
                 h_class_name += ' md_first_part'
         if re.search('<.*?>', content) and not re.search('</.*?>', content):  # header 中有  <??> 但是没有  </xxx> 的进行 escape 的处理
             content = html_escape(content)
-        header_html = '<h%s id="toc_%s" class="%s">%s</h%s>\n' % (level, self.toc_i, h_class_name, content, level)
+        header_html = '<h%s id="toc_%s" class="%s"><span class="span_for_h">%s</span></h%s>\n' % (level, self.toc_i, h_class_name, content, level)
         header_html = smart_tag_style_for_block_html(header_html)
+        header_html = line_patch(header_html)
         self.toc_i += 1
         self.toc_html_parts.append(header_html)
+        if hash_name:
+            header_html += '<a name="%s" class="md_hash_name md_header_hash_name"></a>'%hash_name
         return header_html
 
 
@@ -227,7 +246,14 @@ class FarBoxRenderer(m.HtmlRenderer):
         result = self._blockcode(text, lang)
         return result + '\n<!--block_code_end-->'
 
+    def double_emphasis(self, text):
+        return '<strong class="md_compiled md_compiled_strong">%s</strong>' % text
+
+    def emphasis(self, text):
+        return '<em class="md_compiled md_compiled_em">%s</em>' % text
+
     def codespan(self, text):
+        text = text.strip()
         if '\n' in text:
             # 无 lang，则是 \n 开头
             lang, code_content = text.split('\n', 1)
@@ -237,6 +263,33 @@ class FarBoxRenderer(m.HtmlRenderer):
                 return blockcode_result
         text = html_escape(text)
         text = text.replace('*', '&#42;') # 转义 * 号
+
+        # inline style 的直接处理
+        if self.metadata:
+            maybe_inline_style_name = re.split(u'[:\uff1a]',text, 1)[0]
+            if maybe_inline_style_name:
+                try:
+                    maybe_inline_style_name = 'style_%s' % maybe_inline_style_name.strip()
+                    if maybe_inline_style_name in self.metadata:
+                        inline_style = self.metadata.get(maybe_inline_style_name)
+                        if isinstance(inline_style, string_types):
+                            text = re.split(u'[:\uff1a]',text, 1)[-1].strip()
+                            result = '<span class="inline_style_by_code md_compiled" style="%s">%s</span>' % (inline_style, text)
+                            return result
+                except:
+                    pass
+
+        # 默认用颜色直接构建的 inline style 的处理
+        if re.search(u"[:\uff1a]", text):
+            maybe_inline_text_parts = re.split(u"[:\uff1a]", text, 1)
+            color_key = maybe_inline_text_parts[0].strip().lower()
+            text_to_be_colored = maybe_inline_text_parts[1]
+            font_color = NAMED_COLORS_AND_BACKGROUND.get(color_key)
+            if text_to_be_colored and  font_color:
+                inline_style = "display:inline;color:%s; background:%s; border-radius:5px; padding:5px 12px; margin-right:5px" % (font_color, color_key)
+                result = "<span class=\"inline_style_by_code md_compiled\" style=\"%s\">%s</span>" % (inline_style, text_to_be_colored)
+                return result
+
         result = '<code>%s</code>' % text
         return result
 
@@ -255,12 +308,13 @@ class FarBoxRenderer(m.HtmlRenderer):
             return '\n<pre><code>%s</code></pre>\n' %  escape(text)
 
         # special syntax starts
-        elif lang in ['mathjax']: # 处理数学公式 # <!--random_uuid-->
-            block_html = '\n<section class="mathjax_script" id="Math%s">$$\n%s$$\n</section>\n' % (shortuuid.uuid(), text)
+        elif lang in ['mathjax', 'math']: # 处理数学公式 # <!--random_uuid-->
+            content_md5 = md5(text.strip())
+            block_html = '\n<section class="mathjax_script" id="Math%s" content_id="%s">$$\n%s$$\n</section>\n' % (shortuuid.uuid(), content_md5, text)
             return make_line_chars_as_position_points(block_html, force=True)
 
         elif lang in ['mermaid']: # 处理 mermaid 流程图 # <!--random_uuid-->
-            content_md5 = md5(text)
+            content_md5 = md5(text.strip())
             block_html = '\n<!--js_run_it--><div class="mermaid" id="mermaid_%s" content_id="%s">%s</div>\n'%(shortuuid.uuid(), content_md5, text)
             return make_line_chars_as_position_points(block_html)
 
@@ -286,7 +340,7 @@ class FarBoxRenderer(m.HtmlRenderer):
                 css_class += 'code_lang_%s ' % lang
                 if with_lines:
                     css_class += 'with_lines '
-                css_class += ' highlight'
+                css_class += ' highlight'  # 会叠加成 highlighttable 这样的 class
                 formatter = HtmlFormatter(linenos=with_lines, cssclass=css_class)
                 return highlight(text, lexer, formatter)
             except: # ClassNotFound
@@ -345,7 +399,8 @@ class FarBoxRenderer(m.HtmlRenderer):
             inner_p_class_name += ' p_without_wrap'
 
         for i, line in enumerate(lines):
-            if re.match('\s*<.*?>\s*$', line): # 纯 HTML
+            if not line.startswith('<img ') and re.match('\s*<.*?>\s*$', line) and 'class="md_compiled' not in line:
+                # 纯 HTML & 图片另外处理，增加标识
                 if i == lines_count - 1: # 尾行
                     compiled_html += line
                 else:
@@ -372,7 +427,7 @@ class FarBoxRenderer(m.HtmlRenderer):
             else:
                 has_contents = True
 
-                # [xxxx] 的处理
+                # 单行 [xxxx] 的处理
                 if re.match(r'^\[.*?\]\s*%s?$'%line_char, line):
                     if '[toc]' in line.lower(): # TOC
                         line = line.lower().replace('[toc]', TOC_MARKER)
@@ -395,7 +450,26 @@ class FarBoxRenderer(m.HtmlRenderer):
                 if i == lines_count-1:
                     line_class_names += ' md_line_end'
 
-                line = line_patch(line)
+                line = line_patch(line, compile_wiki_link=True)
+
+                # 高度修饰
+                if re.match("\[-?\d+\]\s*$", line.replace(line_char, "")):
+                    int_value = pick_int(line)
+                    if int_value >= 0:
+                        line = re.sub("\[-?\d+\]", "<div class=\"height\" style=\"height:%spx\"></div>" % int_value, line)
+                    else:
+                        line = re.sub("\[-?\d+\]", "<div class=\"height\" style=\"height:1px;margin-top:%spx\"></div>" % int_value, line)
+                    line_class_names += " md_line_height"
+
+                space_before_line_m = re.match(' +', line)
+                if space_before_line_m: # 保留空格的逻辑
+                    space_chars = space_before_line_m.group()
+                    line = line.lstrip(' ')
+                    space_chars_count = len(space_chars)
+                    space_chars_class = 'md_line_space_chars md_line_space_chars_%s' % space_chars_count
+                    if space_chars_count > 200:
+                        space_chars_class += ' md_line_space_chars_200 md_line_space_chars_200p'
+                    line = '<span class="%s">%s</span>' % (space_chars_class, space_chars) + line
                 line_html = '    <span class="%s">%s%s</span>\n'%(line_class_names, line, line_split_str)
                 line_html = smart_tag_style_for_block_html(line_html)
 
@@ -423,10 +497,18 @@ class FarBoxRenderer(m.HtmlRenderer):
 
 
         if has_contents: # 如果全是注释内容功能，不会占用 dom 节点
+            p_class_name = "md_block"
+            if lines_count == 1:
+                pure_text = re.sub(r'</?[^<]+?>', '', text).replace(line_char, "")
+                if re.search("\\[[^\\^\\[\\]]+\\]$", pure_text): # 清掉 xx [xxx] 的修饰
+                    pure_text = re.sub(u"\\[[^\\^\\[\\]]+\\]$", u"", pure_text).strip()
+                if pure_text and pure_text[-1] in [":", u"："]:
+                    # 约定，单独一行的 p 解析，最后是 ：结尾的，有 md_block_as_opening 这个 class
+                    p_class_name += " md_block_as_opening"
             if inner_p_class_name:
-                compiled_html = '<p class="md_block">\n<section class="%s">\n%s</section>\n</p>' % ( inner_p_class_name, compiled_html)
+                compiled_html = '<p class="%s">\n<section class="%s">\n%s</section>\n</p>' % ( p_class_name, inner_p_class_name, compiled_html)
             else:
-                compiled_html = '<p class="md_block">\n%s</p>' % compiled_html
+                compiled_html = '<p class="%s">\n%s</p>' % (p_class_name, compiled_html)
             compiled_html = '%s\n%s\n%s\n' % (comment_at_start, compiled_html, comment_at_end)
         else:
             compiled_html = comment_at_start + compiled_html + comment_at_end
@@ -436,6 +518,10 @@ class FarBoxRenderer(m.HtmlRenderer):
 
         return compiled_html
 
+    def math(self, text, displaymode):
+        # Parse TeX $$math$$ syntax, Kramdown style
+        return '$$%s$$' % text
+
     def blockquote(self, text):
         md_lines_count = text.count('class="md_line')
         if md_lines_count:
@@ -443,22 +529,43 @@ class FarBoxRenderer(m.HtmlRenderer):
         else:
             blockquote_class_name = ''
         #text = re.sub(r'\n{3,}', '\n</blockquote><blockquote>\n', text)
+        if '<img ' in text:
+            blockquote_class_name += ' blockquote_with_image'
+        else:
+            blockquote_class_name += ' blockquote_without_image'
         if blockquote_class_name:
             result = '\n<blockquote class="%s">%s</blockquote>\n' % (blockquote_class_name, text)
         else:
             result = '\n<blockquote>%s</blockquote>\n' % text
         return result
 
+    def list(self, text, is_ordered, *args):
+        is_ordered_list = is_ordered
+        pure_content = re.sub("^<!--.*?-->", "", text)
+        if is_ordered_list:
+            prefix_info_match = re.match("<!--.*?-->", text)
+            if prefix_info_match:
+                prefix_info = prefix_info_match.group()
+                position_search = re.search("\d+", prefix_info)
+                if position_search:
+                    position = position_search.group()
+                    return "<ol class=\"md_list md_ol\" start=\"%s\">\n%s</ol>\n" % (position, pure_content)
+            return "<ol class=\"md_list md_ol\">\n%s</ol>\n" % pure_content
+        else:
+            return "<ul class=\"md_list md_ul\">\n%s</ul>\n" % pure_content
+
     def listitem(self, text, *args):  #hoedown_list_flags
         # is_ordered, is_block
         #text_strip = text.strip()
         if text.startswith('&nbsp;') and re.search('class=[\'"]__is_wrapped_paragraph[\'"]', text): # 空占位
             return ''
 
-        text = line_patch(text)
-        p_line_match = re.match(r'\n*<p class="md_block">\s*<span class="md_line.*?">([^<>]*?)</span>\s*</p>', text)
+        text = line_patch(text, compile_wiki_link=True)
+        p_line_match = re.match(r'\n*<p class="md_block">\s*<span class="md_line.*?">(.*?)</span>\s*</p>', text)
         if p_line_match: # 嵌套性质的, 所以第一行会被解析为一个 p 元素, 尝试拆掉 第一个 p & span
-            text = text[:p_line_match.start()] + p_line_match.group(1) + text[p_line_match.end():]
+            span_body = p_line_match.group(1)
+            if '<span' not in span_body:
+                text = text[:p_line_match.start()] + span_body + text[p_line_match.end():]
 
         # 空 item 产生的 h1~h6 语法的冲突
         text = re.sub(r'<h[1-6][^<>]*?></h[1-6]>\s*$', '', text)
@@ -470,7 +577,7 @@ class FarBoxRenderer(m.HtmlRenderer):
             text = re.sub(r'\[x\] ', '', text, 1)
             listitem_html = '<li class="md_li todo_item todo_done_item"><input type="checkbox" checked="checked" > %s</li>\n' % text
         else:
-            listitem_html = '<li class="md_li"><span>%s</span></li>\n' % text
+            listitem_html = '<li class="md_li"><span class="md_li_span">%s</span></li>\n' % text
         listitem_html = smart_tag_style_for_block_html(listitem_html)
         return listitem_html
 
@@ -528,10 +635,11 @@ class FarBoxRenderer(m.HtmlRenderer):
         float_style = 0 # 左右浮动
         if "?" in link: # r w h 转为style
             styles = ''
-            w, h, r, rt, float_style = get_get_var(link, ['w', 'h', 'r', 'rt', 'f'])
+            w, h, r, rt, float_style, is_align_center = get_get_var(link, ['w', 'h', 'r', 'rt', 'f', 'c'])
             if w: styles += "max-width:%spx;" % w
             if h: styles += "max-height:%spx;" % h
             if r: styles += "width:%s%%;" % r
+            if is_align_center: styles += 'display:block; margin:0 auto;text-align:center'
             if float_style:
                 if float_style == '1': # left
                     styles += 'display:block;float:left;margin:16px 16px 0 0;'
@@ -554,9 +662,9 @@ class FarBoxRenderer(m.HtmlRenderer):
 
         if rt:
             image_class += ' img_rt_%s'%rt
-        if float_style == 1:
+        if float_style == "1":
             image_class += ' img_float_left'
-        elif float_style == 2:
+        elif float_style == "2":
             image_class += ' img_float_right'
 
         # src 用的是 " 而不是单引号
@@ -565,6 +673,10 @@ class FarBoxRenderer(m.HtmlRenderer):
 
         if self.should_fix_image_paths: # 可以确保图片的地址都是 / 开始的
             compiled_html = fix_images_in_markdown(self.filepath, compiled_html, self.root_path, for_local=self.for_local)
+
+        if alt:
+            compiled_html = '<figure class="md_figure md_image_figure">%s<figcaption>%s</figcaption></figure>' % (compiled_html, alt)
+
         return compiled_html
 
 
@@ -634,14 +746,15 @@ class FarBoxRenderer(m.HtmlRenderer):
 
 def _md_to_html(text, extensions, flags=None, is_toc=False, is_code_file=False,
                 compute_md_blocks=False, wrap_md_blocks=False, keep_lines_between_paragraphs=False,
-                filepath='', root_path='', for_local=False, fix_image_paths=True):
+                filepath='', root_path='', for_local=False, fix_image_paths=True, metadata=None):
     # {%gist 7132248 %} --> <script src='https://gist.github.com/7132175.js'></script>
     # `gist 7132248` works too.
     text = re.sub(r'(?:^|\n)(?:\{%|`) *gist (\w+) *(?:%\}|`)\s*(?:\n|$)', "\n<script src='https://gist.github.com/\g<1>.js'></script>\n", text)
     if not is_toc:
         renderer = FarBoxRenderer(flags, nesting_level=6, is_code_file=is_code_file,
                                   raw_content=text, compute_md_blocks=compute_md_blocks,
-                                  filepath=filepath, root_path=root_path, for_local=for_local, fix_image_paths=fix_image_paths)
+                                  filepath=filepath, root_path=root_path,
+                                  for_local=for_local, fix_image_paths=fix_image_paths, metadata=metadata)
     else:
         renderer = m.HtmlTocRenderer(nesting_level=6) # 没有flags
     markdown = m.Markdown(renderer, extensions)
@@ -658,6 +771,10 @@ def _md_to_html(text, extensions, flags=None, is_toc=False, is_code_file=False,
     if is_toc and html_content == '</li>\n</ul>\n':
         html_content = ''
 
+    # 尾段
+    html_content = rreplace(html_content, '<p class="md_block">', '<p class="md_block last_md_block_in_page">')
+    wrapped_html = rreplace(wrapped_html, '<p class="md_block">', '<p class="md_block last_md_block_in_page">')
+
     html_content = UnicodeWithAttrs(html_content)
     html_content.blocks_info = blocks_info
     html_content.html_blocks = html_blocks
@@ -667,9 +784,11 @@ def _md_to_html(text, extensions, flags=None, is_toc=False, is_code_file=False,
 
 
 
-def line_patch(line):
+def line_patch(line, compile_wiki_link=False):
     # 由于 cjk 的关系, 比如** * ~~ 在没有空格的时候,未必有准确的对应
     # 但是前提是前、后不能跟英文、数字
+    if compile_wiki_link:
+        line = compile_markdown_wiki_link_syntax(line)
     line = re.sub('([^a-z0-9~]|\A)~~([^~<>]*?)~~([^a-z0-9~]|\Z)','\g<1><del>\g<2></del>\g<3>', line, flags=re.I) # 删除符的支持
     line = re.sub('(?<!\*)\*\*([^\*<>]*?)?\*\*(?!\*)','<strong>\g<1></strong>', line, flags=re.I) # 加粗
     line = patch_em(line)
@@ -735,7 +854,8 @@ def blockquote_fix_man(match, code_block_positions):
     return '%s%s' % (part_one, joiner.replace('>', '>&nbsp;'))
 
 def markdown_to_html(content, old_markdown=False, no_html=False, toc=True, footnotes=True, is_code_file=False,
-                     compute_md_blocks=False, wrap_md_blocks=False, filepath='', root_path='', for_local=False, fix_image_paths=True):
+                     compute_md_blocks=False, wrap_md_blocks=False, filepath='', root_path='',
+                     for_local=False, fix_image_paths=True, metadata=None):
 
     # 内容的预处理
     content = content.replace('\r\n', '\n').replace('\t', '    ')
@@ -771,7 +891,9 @@ def markdown_to_html(content, old_markdown=False, no_html=False, toc=True, footn
 
     content = _md_to_html(raw_content, extensions=extensions, flags=flags, is_code_file=is_code_file,
                           compute_md_blocks=compute_md_blocks, wrap_md_blocks=wrap_md_blocks,
-                          filepath=filepath, root_path=root_path, for_local=for_local, fix_image_paths=fix_image_paths)
+                          filepath=filepath, root_path=root_path, for_local=for_local, fix_image_paths=fix_image_paths,
+                          metadata = metadata
+                          )
     blocks_info = getattr(content, 'blocks_info', {})
     html_blocks = getattr(content, 'html_blocks', {})
     wrapped_html = getattr(content, 'wrapped_html', '')
@@ -845,7 +967,7 @@ def replace_toc_marker(content, toc_content):
 # 对外调用的主函数
 def compile_markdown(content, path='/', remove_h1=True, toc=False, root=None,
                      compute_md_blocks=False, wrap_md_blocks=False, for_local=False,
-                     compile_metadata=True, do_after_job=True, fix_image_paths=True):
+                     compile_metadata=True, do_after_job=True, fix_image_paths=True, metadata=None):
     # 约0.008s处理一篇普通大小的文档, replace 基本上没有太大性能损耗，主要是正则处理比较复杂的时候性能会有损耗
     # 去除了markdown文档头部信息, 必须保证是头部...
     # path是指文档的路径，有些相对路径，特别是web类的应用，其它情况下则是文档的绝对路径
@@ -864,8 +986,10 @@ def compile_markdown(content, path='/', remove_h1=True, toc=False, root=None,
     if compile_metadata:
         content, metadata = extract_metadata(content)  # 先做jekyll的兼容
     else:
-        # 也有可能不做 metadata 的处置逻辑
-        metadata = {}
+        # 也有可能不做 metadata 的处置逻辑, 比如局部更新
+        metadata = metadata or {}
+        if not isinstance(metadata, dict):
+            metadata = {}
 
     clean_raw_content = content  # 这个content是没有metadata内容了的
 
@@ -899,7 +1023,7 @@ def compile_markdown(content, path='/', remove_h1=True, toc=False, root=None,
     content, toc_content = markdown_to_html(content, old_markdown, is_code_file=is_code_file,
                                             compute_md_blocks=compute_md_blocks, wrap_md_blocks=wrap_md_blocks,
                                             filepath=path, root_path=root, for_local=for_local,
-                                            fix_image_paths=fix_image_paths,
+                                            fix_image_paths=fix_image_paths, metadata=metadata
                                             )
 
     if not do_after_job:
@@ -922,6 +1046,7 @@ def compile_markdown(content, path='/', remove_h1=True, toc=False, root=None,
     h1_removed = False
     content = content.strip()
     h1s = re.findall(r'(<h1[^<]*?>(.*?)</h1>)', content, re.I)
+    title_from_h1 = False
     if metadata.get('title'):
         title = metadata.get('title')
         title_from_post = True
@@ -932,14 +1057,15 @@ def compile_markdown(content, path='/', remove_h1=True, toc=False, root=None,
             content = content.replace(h1s[0][0], '', 1).strip()
             h1_removed = True
         title_from_post = True
+        title_from_h1 = True
     else:
         title = os.path.split(path)[1].rsplit('.', 1)[0]
         title = re.sub(r'^\d+(\.\d+)? ', '', title) # 去掉可能的序号
 
     # ul/ol 上层 p 的 class 锚定
     # for p_item in re.findall(r'(<p class="md_block">)(.*?</p>\s*<(\w+)[> ])', content, re.M|re.S):
-    content = re.sub(r'(<p class="md_block">)((?:(?!</p>).*\n)+?)(</p>[\s\r\n]*<(ul|ol|blockquote|img|div class="codehilite)[> ])',
-                     p_bellow_block_replacer, content)
+    #content = re.sub(r'(<p class="md_block">)((?:(?!</p>).*\n)+?)(</p>[\s\r\n]*<(ul|ol|blockquote|img|div class="codehilite)[> ])',
+    #                 p_bellow_block_replacer, content)
 
     # toc
     if toc:
@@ -963,6 +1089,9 @@ def compile_markdown(content, path='/', remove_h1=True, toc=False, root=None,
         #if isinstance(css_content, string_types):
             #content = '<style>%s</style>\n%s' % (css_content, content)
 
+    metadata['title_from_post'] = title_from_post
+    metadata['title_from_h1'] = title_from_h1
+
     content = UnicodeWithAttrs(content)
     content.clean = clean_raw_content
     content.metadata = metadata
@@ -970,6 +1099,7 @@ def compile_markdown(content, path='/', remove_h1=True, toc=False, root=None,
     content.toc = toc_content
     content.title = title
     content.title_from_post = title_from_post
+    content.title_from_h1 = title_from_h1
     content.html_blocks = html_blocks
     content.wrapped_html = wrapped_html
 
